@@ -1,5 +1,7 @@
 //! This module contains implementations of `Dummy` trait
 
+#[cfg(feature = "base64")]
+pub mod base64;
 #[cfg(feature = "bigdecimal-rs")]
 pub mod bigdecimal;
 #[cfg(feature = "bson_oid")]
@@ -12,6 +14,8 @@ pub mod chrono_tz;
 pub mod color;
 #[cfg(feature = "rust_decimal")]
 pub mod decimal;
+#[cfg(feature = "email_address")]
+pub mod email_address;
 #[cfg(feature = "geo-types")]
 pub mod geo;
 #[cfg(feature = "glam")]
