@@ -53,9 +53,9 @@ use itertools::Itertools;
 pub use link::{link_file, LinkFileError, LinkMethod};
 pub use python::PythonInfo;
 use rattler_conda_types::{
-    package::{IndexJson, LinkJson, NoArchLinks, PackageFile, PathsJson},
-    prefix_record::PathsEntry,
-    Platform,
+    package::{IndexJson, LinkJson, NoArchLinks, PackageFile, PathsEntry, PathsJson},
+    prefix::Prefix,
+    prefix_record, Platform,
 };
 use rayon::{
     iter::Either,
@@ -248,6 +248,12 @@ pub struct InstallOptions {
     pub apple_codesign_behavior: AppleCodeSignBehavior,
 }
 
+struct LinkPath {
+    entry: PathsEntry,
+    computed_path: PathBuf,
+    clobber_path: Option<PathBuf>,
+}
+
 /// Given an extracted package archive (`package_dir`), installs its files to
 /// the `target_dir`.
 ///
@@ -257,10 +263,10 @@ pub struct InstallOptions {
 #[instrument(skip_all, fields(package_dir = % package_dir.display()))]
 pub async fn link_package(
     package_dir: &Path,
-    target_dir: &Path,
+    target_dir: &Prefix,
     driver: &InstallDriver,
     options: InstallOptions,
-) -> Result<Vec<PathsEntry>, InstallError> {
+) -> Result<Vec<prefix_record::PathsEntry>, InstallError> {
     // Determine the target prefix for linking
     let target_prefix = options
         .target_prefix
@@ -270,11 +276,6 @@ pub async fn link_package(
         .ok_or(InstallError::TargetPrefixIsNotUtf8)?
         .to_owned();
 
-    // Ensure target directory exists
-    tokio_fs::create_dir_all(&target_dir)
-        .await
-        .map_err(InstallError::FailedToCreateTargetDirectory)?;
-
     // Reuse or read the `paths.json` and `index.json` files from the package
     // directory
     let paths_json = read_paths_json(package_dir, driver, options.paths_json);
@@ -319,7 +320,7 @@ pub async fn link_package(
     let platform = options.platform.unwrap_or(Platform::current());
 
     // compute all path renames
-    let mut final_paths = compute_paths(&index_json, &paths_json, options.python_info.as_ref());
+    let final_paths = compute_paths(&index_json, &paths_json, options.python_info.as_ref());
 
     // register all paths in the install driver path registry
     let clobber_paths = Arc::new(
@@ -328,16 +329,23 @@ pub async fn link_package(
             .register_paths(&index_json, &final_paths),
     );
 
-    for (_, computed_path) in final_paths.iter_mut() {
-        if let Some(clobber_rename) = clobber_paths.get(computed_path) {
-            *computed_path = clobber_rename.clone();
-        }
-    }
+    let final_paths: Vec<LinkPath> = final_paths
+        .into_iter()
+        .map(|el| {
+            let (entry, computed_path) = el;
+            let clobber_path = clobber_paths.get(&computed_path).cloned();
+            LinkPath {
+                entry,
+                computed_path,
+                clobber_path,
+            }
+        })
+        .collect();
 
     // Figure out all the directories that we are going to need
     let mut directories_to_construct = HashSet::new();
-    for (_, computed_path) in final_paths.iter() {
-        let mut current_path = computed_path.parent();
+    for link_path in &final_paths {
+        let mut current_path = link_path.computed_path.parent();
         while let Some(path) = current_path {
             if !path.as_os_str().is_empty() && directories_to_construct.insert(path.to_path_buf()) {
                 current_path = path.parent();
@@ -347,7 +355,7 @@ pub async fn link_package(
         }
     }
 
-    let directories_target_dir = target_dir.to_path_buf();
+    let directories_target_dir = target_dir.path().to_path_buf();
     driver
         .run_blocking_io_task(move || {
             for directory in directories_to_construct.into_iter().sorted() {
@@ -369,12 +377,12 @@ pub async fn link_package(
     // Start linking all package files in parallel
     let mut pending_futures = FuturesUnordered::new();
     let mut number_of_paths_entries = 0;
-    for (entry, computed_path) in final_paths {
+    for link_path in final_paths {
+        let entry = link_path.entry;
         let package_dir = package_dir.to_owned();
         let target_dir = target_dir.to_owned();
         let target_prefix = target_prefix.clone();
 
-        let clobber_rename = clobber_paths.get(&entry.relative_path).cloned();
         let install_future = async move {
             let _permit = driver.acquire_io_permit().await;
 
@@ -382,10 +390,11 @@ pub async fn link_package(
             // because filesystem access is blocking anyway so its more
             // efficient to group them together in a single blocking call.
             let cloned_entry = entry.clone();
+            let is_clobber = link_path.clobber_path.is_some();
             let result = match tokio::task::spawn_blocking(move || {
                 link_file(
                     &cloned_entry,
-                    computed_path,
+                    link_path.clobber_path.unwrap_or(link_path.computed_path),
                     &package_dir,
                     &target_dir,
                     &target_prefix,
@@ -408,9 +417,9 @@ pub async fn link_package(
             };
 
             // Construct a `PathsEntry` from the result of the linking operation
-            let paths_entry = PathsEntry {
+            let paths_entry = prefix_record::PathsEntry {
                 relative_path: result.relative_path,
-                original_path: if clobber_rename.is_some() {
+                original_path: if is_clobber {
                     Some(entry.relative_path)
                 } else {
                     None
@@ -513,7 +522,8 @@ pub async fn link_package(
     // What makes this loop special is that it also aborts if any of the returned
     // results indicate a failure.
     let mut paths = Vec::with_capacity(number_of_paths_entries);
-    let mut out_of_order_queue = BinaryHeap::<OrderWrapper<PathsEntry>>::with_capacity(100);
+    let mut out_of_order_queue =
+        BinaryHeap::<OrderWrapper<prefix_record::PathsEntry>>::with_capacity(100);
     while let Some(link_result) = pending_futures.next().await {
         for (index, data) in link_result? {
             if index == paths.len() {
@@ -557,10 +567,10 @@ pub async fn link_package(
 #[instrument(skip_all, fields(package_dir = % package_dir.display()))]
 pub fn link_package_sync(
     package_dir: &Path,
-    target_dir: &Path,
+    target_dir: &Prefix,
     clobber_registry: Arc<Mutex<ClobberRegistry>>,
     options: InstallOptions,
-) -> Result<Vec<PathsEntry>, InstallError> {
+) -> Result<Vec<prefix_record::PathsEntry>, InstallError> {
     // Determine the target prefix for linking
     let target_prefix = options
         .target_prefix
@@ -570,9 +580,6 @@ pub fn link_package_sync(
         .ok_or(InstallError::TargetPrefixIsNotUtf8)?
         .to_owned();
 
-    // Ensure target directory exists
-    fs_err::create_dir_all(target_dir).map_err(InstallError::FailedToCreateTargetDirectory)?;
-
     // Reuse or read the `paths.json` and `index.json` files from the package
     // directory
     let paths_json = options.paths_json.map_or_else(
@@ -639,7 +646,7 @@ pub fn link_package_sync(
     let platform = options.platform.unwrap_or(Platform::current());
 
     // compute all path renames
-    let mut final_paths = compute_paths(&index_json, &paths_json, options.python_info.as_ref());
+    let final_paths = compute_paths(&index_json, &paths_json, options.python_info.as_ref());
 
     // register all paths in the install driver path registry
     let clobber_paths = clobber_registry
@@ -647,17 +654,21 @@ pub fn link_package_sync(
         .unwrap()
         .register_paths(&index_json, &final_paths);
 
-    for (_, computed_path) in final_paths.iter_mut() {
-        if let Some(clobber_rename) = clobber_paths.get(computed_path) {
-            *computed_path = clobber_rename.clone();
+    let final_paths = final_paths.into_iter().map(|el| {
+        let (entry, computed_path) = el;
+        let clobber_path = clobber_paths.get(&computed_path).cloned();
+        LinkPath {
+            entry,
+            computed_path,
+            clobber_path,
         }
-    }
+    });
 
     // Figure out all the directories that we are going to need
     let mut directories_to_construct = HashSet::new();
     let mut paths_by_directory = HashMap::new();
-    for (entry, computed_path) in final_paths {
-        let Some(entry_parent) = computed_path.parent() else {
+    for link_path in final_paths {
+        let Some(entry_parent) = link_path.computed_path.parent() else {
             continue;
         };
 
@@ -675,7 +686,7 @@ pub fn link_package_sync(
         paths_by_directory
             .entry(entry_parent.to_path_buf())
             .or_insert_with(Vec::new)
-            .push((entry, computed_path));
+            .push(link_path);
     }
 
     let mut created_directories = HashSet::new();
@@ -684,7 +695,7 @@ pub fn link_package_sync(
         .into_iter()
         .sorted_by(|a, b| a.components().count().cmp(&b.components().count()))
     {
-        let full_path = target_dir.join(&directory);
+        let full_path = target_dir.path().join(&directory);
 
         // if we already (recursively) created the parent directory we can skip this
         if created_directories
@@ -696,9 +707,10 @@ pub fn link_package_sync(
 
         // can we lock this directory?
         if full_path.exists() {
-            // tracing::info!("directory already exists: {:?}", directory);
             continue;
-        } else if allow_ref_links && cfg!(target_os = "macos") && !index_json.noarch.is_python() {
+        }
+
+        if allow_ref_links && cfg!(target_os = "macos") && !index_json.noarch.is_python() {
             // reflink the whole directory if possible
             // currently this does not handle noarch packages
             match reflink_copy::reflink(package_dir.join(&directory), &full_path) {
@@ -729,32 +741,33 @@ pub fn link_package_sync(
 
     // Take care of all the reflinked files (macos only)
     //  - Add them to the paths.json
-    //  - Fix any occurences of the prefix in the files
+    //  - Fix any occurrences of the prefix in the files
     //  - Rename files that need clobber-renames
     let mut reflinked_paths_entries = Vec::new();
     for (parent_dir, files) in reflinked_files {
         // files that are either in the clobber map or contain a placeholder,
         // we defer to the regular linking that comes after this block
         // and re-add them to the paths_by_directory map
-        for file in files {
-            if clobber_paths.contains_key(&file.1) || file.0.prefix_placeholder.is_some() {
+        for link_path in files {
+            if link_path.clobber_path.is_some() || link_path.entry.prefix_placeholder.is_some() {
                 paths_by_directory
                     .entry(parent_dir.clone())
                     .or_insert_with(Vec::new)
-                    .push(file);
+                    .push(link_path);
             } else {
-                reflinked_paths_entries.push(PathsEntry {
-                    relative_path: file.0.relative_path,
-                    path_type: file.0.path_type.into(),
-                    no_link: file.0.no_link,
-                    sha256: file.0.sha256,
-                    size_in_bytes: file.0.size_in_bytes,
+                let entry = link_path.entry;
+                reflinked_paths_entries.push(prefix_record::PathsEntry {
+                    relative_path: entry.relative_path,
+                    path_type: entry.path_type.into(),
+                    no_link: entry.no_link,
+                    sha256: entry.sha256,
+                    size_in_bytes: entry.size_in_bytes,
                     // No placeholder, no clobbering, so these are none for sure
                     original_path: None,
                     sha256_in_prefix: None,
                     file_mode: None,
                     prefix_placeholder: None,
-                })
+                });
             }
         }
     }
@@ -766,7 +779,6 @@ pub fn link_package_sync(
     // Link the individual files in parallel
     let link_target_prefix = target_prefix.clone();
     let package_dir = package_dir.to_path_buf();
-    let link_target_dir = target_dir.to_path_buf();
     let mut paths = paths_by_directory
         .into_values()
         .collect_vec()
@@ -774,14 +786,17 @@ pub fn link_package_sync(
         .with_min_len(100)
         .flat_map(move |entries_in_subdir| {
             let mut path_entries = Vec::with_capacity(entries_in_subdir.len());
-            for (entry, computed_path) in entries_in_subdir {
-                let clobber_rename = clobber_paths.get(&entry.relative_path).cloned();
+            for link_path in entries_in_subdir {
+                let entry = link_path.entry;
 
+                let is_clobber = link_path.clobber_path.is_some();
                 let link_result = link_file(
                     &entry,
-                    computed_path.clone(),
+                    link_path
+                        .clobber_path
+                        .unwrap_or(link_path.computed_path.clone()),
                     &package_dir,
-                    &link_target_dir,
+                    target_dir,
                     &link_target_prefix,
                     allow_symbolic_links && !entry.no_link,
                     allow_hard_links && !entry.no_link,
@@ -801,10 +816,10 @@ pub fn link_package_sync(
                 };
 
                 // Construct a `PathsEntry` from the result of the linking operation
-                path_entries.push(Ok(PathsEntry {
+                path_entries.push(Ok(prefix_record::PathsEntry {
                     relative_path: result.relative_path,
-                    original_path: if clobber_rename.is_some() {
-                        Some(entry.relative_path.clone())
+                    original_path: if is_clobber {
+                        Some(link_path.computed_path)
                     } else {
                         None
                     },
@@ -850,7 +865,6 @@ pub fn link_package_sync(
             .expect("should be safe because its checked above that this contains a value");
 
         let target_prefix = target_prefix.clone();
-        let target_dir = target_dir.to_path_buf();
 
         // Create entry points for each listed item. This is different between Windows
         // and unix because on Windows, two PathEntry's are created whereas on
@@ -862,7 +876,7 @@ pub fn link_package_sync(
                 // .with_min_len(100)
                 .flat_map(move |entry_point| {
                     match create_windows_python_entry_point(
-                        &target_dir,
+                        target_dir,
                         &target_prefix,
                         &entry_point,
                         &python_info,
@@ -882,7 +896,7 @@ pub fn link_package_sync(
                 // .with_min_len(100)
                 .map(move |entry_point| {
                     match create_unix_python_entry_point(
-                        &target_dir,
+                        target_dir,
                         &target_prefix,
                         &entry_point,
                         &python_info,
@@ -993,9 +1007,9 @@ async fn read_link_json(
 }
 
 /// Returns true if it is possible to create symlinks in the target directory.
-fn can_create_symlinks_sync(target_dir: &Path) -> bool {
+fn can_create_symlinks_sync(target_dir: &Prefix) -> bool {
     let uuid = uuid::Uuid::new_v4();
-    let symlink_path = target_dir.join(format!("symtest_{uuid}"));
+    let symlink_path = target_dir.path().join(format!("symtest_{uuid}"));
     #[cfg(windows)]
     let result = std::os::windows::fs::symlink_file("./", &symlink_path);
     #[cfg(unix)]
@@ -1048,9 +1062,9 @@ impl<T> Ord for OrderWrapper<T> {
 }
 
 /// Returns true if it is possible to create symlinks in the target directory.
-async fn can_create_symlinks(target_dir: &Path) -> bool {
+async fn can_create_symlinks(target_dir: &Prefix) -> bool {
     let uuid = uuid::Uuid::new_v4();
-    let symlink_path = target_dir.join(format!("symtest_{uuid}"));
+    let symlink_path = target_dir.path().join(format!("symtest_{uuid}"));
     #[cfg(windows)]
     let result = tokio_fs::symlink_file("./", &symlink_path).await;
     #[cfg(unix)]
@@ -1076,21 +1090,21 @@ async fn can_create_symlinks(target_dir: &Path) -> bool {
 
 /// Returns true if it is possible to create hard links from the target
 /// directory to the package cache directory.
-async fn can_create_hardlinks(target_dir: &Path, package_dir: &Path) -> bool {
+async fn can_create_hardlinks(target_dir: &Prefix, package_dir: &Path) -> bool {
     paths_have_same_filesystem(target_dir, package_dir).await
 }
 
 /// Returns true if it is possible to create hard links from the target
 /// directory to the package cache directory.
-fn can_create_hardlinks_sync(target_dir: &Path, package_dir: &Path) -> bool {
-    paths_have_same_filesystem_sync(target_dir, package_dir)
+fn can_create_hardlinks_sync(target_dir: &Prefix, package_dir: &Path) -> bool {
+    paths_have_same_filesystem_sync(target_dir.path(), package_dir)
 }
 
 /// Returns true if two paths share the same filesystem
 #[cfg(unix)]
-async fn paths_have_same_filesystem(a: &Path, b: &Path) -> bool {
+async fn paths_have_same_filesystem(a: &Prefix, b: &Path) -> bool {
     use std::os::unix::fs::MetadataExt;
-    match tokio::join!(tokio_fs::metadata(a), tokio_fs::metadata(b)) {
+    match tokio::join!(tokio_fs::metadata(a.path()), tokio_fs::metadata(b)) {
         (Ok(a), Ok(b)) => a.dev() == b.dev(),
         _ => false,
     }
@@ -1140,7 +1154,7 @@ mod test {
 
     use crate::{
         get_test_data_dir,
-        install::{link_package, InstallDriver, InstallOptions, PythonInfo},
+        install::{link_package, InstallDriver, InstallOptions, Prefix, PythonInfo},
         package_cache::PackageCache,
     };
 
@@ -1208,13 +1222,14 @@ mod test {
         // Download and install each layer into an environment.
         let install_driver = InstallDriver::default();
         let target_dir = tempdir().unwrap();
+        let prefix_path = Prefix::create(target_dir.path()).unwrap();
         stream::iter(urls)
             .for_each_concurrent(Some(50), |package_url| {
-                let prefix_path = target_dir.path();
                 let client = client.clone();
                 let package_cache = &package_cache;
                 let install_driver = &install_driver;
                 let python_version = &python_version;
+                let prefix_path = prefix_path.clone();
                 async move {
                     // Populate the cache
                     let package_info = ArchiveIdentifier::try_from_url(&package_url).unwrap();
@@ -1231,7 +1246,7 @@ mod test {
                     // Install the package to the prefix
                     link_package(
                         package_cache_lock.path(),
-                        prefix_path,
+                        &prefix_path,
                         install_driver,
                         InstallOptions {
                             python_info: Some(python_version.clone()),
@@ -1285,7 +1300,7 @@ mod test {
         // Link the package
         let paths = link_package(
             package_dir.path(),
-            environment_dir.path(),
+            &Prefix::create(environment_dir.path()).unwrap(),
             &install_driver,
             InstallOptions::default(),
         )
