@@ -8,6 +8,7 @@ mod channel;
 mod channel_data;
 mod explicit_environment_spec;
 mod match_spec;
+pub mod menuinst;
 mod no_arch_type;
 mod parse_mode;
 mod platform;
@@ -22,12 +23,13 @@ mod environment_yaml;
 mod generic_virtual_package;
 pub mod package;
 mod package_name;
+pub mod prefix;
 pub mod prefix_record;
 
 #[cfg(test)]
 use std::path::{Path, PathBuf};
 
-pub use build_spec::{BuildNumber, BuildNumberSpec, ParseBuildNumberSpecError};
+pub use build_spec::{BuildNumber, BuildNumberSpec, OrdOperator, ParseBuildNumberSpecError};
 pub use channel::{Channel, ChannelConfig, ChannelUrl, NamedChannelOrUrl, ParseChannelError};
 pub use channel_data::{ChannelData, ChannelDataPackage};
 pub use environment_yaml::{EnvironmentYaml, MatchSpecOrSubSection};
@@ -41,7 +43,7 @@ pub use match_spec::{
     parse::ParseMatchSpecError,
     MatchSpec, Matches, NamelessMatchSpec,
 };
-pub use no_arch_type::{NoArchKind, NoArchType};
+pub use no_arch_type::{NoArchKind, NoArchType, RawNoArchType};
 pub use package_name::{InvalidPackageNameError, PackageName};
 pub use parse_mode::ParseStrictness;
 pub use platform::{Arch, ParseArchError, ParsePlatformError, Platform};
@@ -53,7 +55,7 @@ pub use repo_data::{
     ChannelInfo, ConvertSubdirError, PackageRecord, RecordFromPath, RepoData,
     ValidatePackageRecordsError,
 };
-pub use repo_data_record::RepoDataRecord;
+pub use repo_data_record::{RepoDataRecord, SolverResult};
 pub use run_export::RunExportKind;
 pub use version::{
     Component, ParseVersionError, ParseVersionErrorKind, StrictVersion, Version, VersionBumpError,
