@@ -6,7 +6,7 @@ pub mod sharded;
 mod topological_sort;
 
 use std::{
-    collections::BTreeSet,
+    collections::{BTreeMap, BTreeSet},
     fmt::{Display, Formatter},
     path::Path,
 };
@@ -15,7 +15,7 @@ use fxhash::{FxHashMap, FxHashSet};
 use rattler_digest::{serde::SerializableHash, Md5Hash, Sha256Hash};
 use rattler_macros::sorted;
 use serde::{Deserialize, Serialize};
-use serde_with::{serde_as, skip_serializing_none, OneOrMany};
+use serde_with::{serde_as, skip_serializing_none};
 use thiserror::Error;
 use url::Url;
 
@@ -26,7 +26,7 @@ use crate::{
         serde::{sort_map_alphabetically, DeserializeFromStrUnchecked},
         UrlWithTrailingSlash,
     },
-    Channel, MatchSpec, Matches, NoArchType, PackageName, PackageUrl, ParseMatchSpecError,
+    Arch, Channel, MatchSpec, Matches, NoArchType, PackageName, PackageUrl, ParseMatchSpecError,
     ParseStrictness, Platform, RepoDataRecord, VersionWithSource,
 };
 
@@ -71,7 +71,7 @@ pub struct RepoData {
 #[derive(Debug, Deserialize, Serialize, Eq, PartialEq, Clone)]
 pub struct ChannelInfo {
     /// The channel's subdirectory
-    pub subdir: String,
+    pub subdir: Option<String>,
 
     /// The `base_url` for all package urls. Can be an absolute or relative url.
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -93,7 +93,10 @@ pub trait RecordFromPath {
 #[sorted]
 #[derive(Debug, Deserialize, Serialize, Eq, PartialEq, Clone, Hash)]
 pub struct PackageRecord {
-    /// Optionally the architecture the package supports
+    /// Optionally the architecture the package supports. This is almost
+    /// always the second part of the `subdir` string. Except for `64` which
+    /// maps to `x86_64` and `32` which maps to `x86`. This will be `None` if
+    /// the package is `noarch`.
     pub arch: Option<String>,
 
     /// The build string of the package
@@ -114,6 +117,12 @@ pub struct PackageRecord {
     #[serde(default)]
     pub depends: Vec<String>,
 
+    /// Specifications of optional or dependencies. These are dependencies that
+    /// are only required if certain features are enabled or if certain
+    /// conditions are met.
+    #[serde(default, skip_serializing_if = "BTreeMap::is_empty")]
+    pub extra_depends: BTreeMap<String, Vec<String>>,
+
     /// Features are a deprecated way to specify different feature sets for the
     /// conda solver. This is not supported anymore and should not be used.
     /// Instead, `mutex` packages should be used to specify
@@ -146,8 +155,11 @@ pub struct PackageRecord {
     #[serde(skip_serializing_if = "NoArchType::is_none")]
     pub noarch: NoArchType,
 
-    /// Optionally the platform the package supports
-    pub platform: Option<String>, // Note that this does not match the [`Platform`] enum..
+    /// Optionally the platform the package supports.
+    /// Note that this does not match the [`Platform`] enum, but is only the first
+    /// part of the platform (e.g. `linux`, `osx`, `win`, ...).
+    /// The `subdir` field contains the `Platform` enum.
+    pub platform: Option<String>,
 
     /// Package identifiers of packages that are equivalent to this package but
     /// from other ecosystems.
@@ -186,11 +198,10 @@ pub struct PackageRecord {
     pub timestamp: Option<chrono::DateTime<chrono::Utc>>,
 
     /// Track features are nowadays only used to downweight packages (ie. give
-    /// them less priority). To that effect, the number of track features is
-    /// counted (number of commas) and the package is downweighted
+    /// them less priority). To that effect, the package is downweighted
     /// by the number of track_features.
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
-    #[serde_as(as = "OneOrMany<_>")]
+    #[serde_as(as = "crate::utils::serde::Features")]
     pub track_features: Vec<String>,
 
     /// The version of the package
@@ -323,6 +334,7 @@ impl PackageRecord {
             noarch: NoArchType::default(),
             platform: None,
             python_site_packages_path: None,
+            extra_depends: BTreeMap::new(),
             sha256: None,
             size: None,
             subdir: Platform::current().to_string(),
@@ -391,6 +403,7 @@ impl PackageRecord {
 
 /// An error when validating package records.
 #[derive(Debug, Error)]
+#[allow(clippy::large_enum_variant)]
 pub enum ValidatePackageRecordsError {
     /// A package is not present in the environment.
     #[error("package '{package}' has dependency '{dependency}', which is not in the environment")]
@@ -451,33 +464,17 @@ fn determine_subdir(
     let platform = platform.ok_or(ConvertSubdirError::PlatformEmpty)?;
     let arch = arch.ok_or(ConvertSubdirError::ArchEmpty)?;
 
-    let plat = match platform.as_ref() {
-        "linux" => match arch.as_ref() {
-            "x86" => Ok(Platform::Linux32),
-            "x86_64" => Ok(Platform::Linux64),
-            "aarch64" => Ok(Platform::LinuxAarch64),
-            "armv61" => Ok(Platform::LinuxArmV6l),
-            "armv71" => Ok(Platform::LinuxArmV7l),
-            "ppc64le" => Ok(Platform::LinuxPpc64le),
-            "ppc64" => Ok(Platform::LinuxPpc64),
-            "s390x" => Ok(Platform::LinuxS390X),
-            _ => Err(ConvertSubdirError::NoKnownCombination { platform, arch }),
-        },
-        "osx" => match arch.as_ref() {
-            "x86_64" => Ok(Platform::Osx64),
-            "arm64" => Ok(Platform::OsxArm64),
-            _ => Err(ConvertSubdirError::NoKnownCombination { platform, arch }),
-        },
-        "windows" => match arch.as_ref() {
-            "x86" => Ok(Platform::Win32),
-            "x86_64" => Ok(Platform::Win64),
-            "arm64" => Ok(Platform::WinArm64),
-            _ => Err(ConvertSubdirError::NoKnownCombination { platform, arch }),
-        },
-        _ => Err(ConvertSubdirError::NoKnownCombination { platform, arch }),
-    }?;
-    // Convert back to Platform string which should correspond to known subdirs
-    Ok(plat.to_string())
+    match arch.parse::<Arch>() {
+        Ok(arch) => {
+            let arch_str = match arch {
+                Arch::X86 => "32",
+                Arch::X86_64 => "64",
+                _ => arch.as_str(),
+            };
+            Ok(format!("{platform}-{arch_str}"))
+        }
+        Err(_) => Err(ConvertSubdirError::NoKnownCombination { platform, arch }),
+    }
 }
 
 impl PackageRecord {
@@ -511,6 +508,7 @@ impl PackageRecord {
             noarch: index.noarch,
             platform: index.platform,
             python_site_packages_path: index.python_site_packages_path,
+            extra_depends: BTreeMap::new(),
             sha256,
             size,
             subdir,
