@@ -87,9 +87,11 @@ mod channel;
 mod conda;
 mod file_format_version;
 mod hash;
+pub mod options;
 mod parse;
 mod pypi;
 mod pypi_indexes;
+pub mod source;
 mod url_or_path;
 mod utils;
 
@@ -98,6 +100,7 @@ pub use channel::Channel;
 pub use conda::{CondaBinaryData, CondaPackageData, CondaSourceData, ConversionError, InputHash};
 pub use file_format_version::FileFormatVersion;
 pub use hash::PackageHashes;
+pub use options::SolveOptions;
 pub use parse::ParseCondaLockError;
 pub use pypi::{PypiPackageData, PypiPackageEnvironmentData, PypiSourceTreeHashable};
 pub use pypi_indexes::{FindLinksUrlOrPath, PypiIndexes};
@@ -157,6 +160,9 @@ struct EnvironmentData {
     /// The pypi indexes used to solve the environment.
     indexes: Option<PypiIndexes>,
 
+    /// The options that were used to solve the environment.
+    options: SolveOptions,
+
     /// For each individual platform this environment supports we store the
     /// package identifiers associated with the environment.
     packages: FxHashMap<Platform, IndexSet<EnvironmentPackageData>>,
@@ -185,14 +191,12 @@ impl LockFile {
     /// Writes the conda lock to a file
     pub fn to_path(&self, path: &Path) -> Result<(), std::io::Error> {
         let file = std::fs::File::create(path)?;
-        serde_yaml::to_writer(file, self)
-            .map_err(|err| std::io::Error::new(std::io::ErrorKind::Other, err))
+        serde_yaml::to_writer(file, self).map_err(std::io::Error::other)
     }
 
     /// Writes the conda lock to a string
     pub fn render_to_string(&self) -> Result<String, std::io::Error> {
-        serde_yaml::to_string(self)
-            .map_err(|err| std::io::Error::new(std::io::ErrorKind::Other, err))
+        serde_yaml::to_string(self).map_err(std::io::Error::other)
     }
 
     /// Returns the environment with the given name.
@@ -230,6 +234,11 @@ impl LockFile {
     pub fn version(&self) -> FileFormatVersion {
         self.inner.version
     }
+
+    /// Check if there are any packages in the lockfile
+    pub fn is_empty(&self) -> bool {
+        self.inner.conda_packages.is_empty() && self.inner.pypi_packages.is_empty()
+    }
 }
 
 /// Information about a specific environment in the lock-file.
@@ -272,6 +281,11 @@ impl<'lock> Environment<'lock> {
         self.data().indexes.as_ref()
     }
 
+    /// Returns the solver options that were used to create this environment.
+    pub fn solve_options(&self) -> &SolveOptions {
+        &self.data().options
+    }
+
     /// Returns all the packages for a specific platform in this environment.
     pub fn packages(
         &self,
@@ -419,6 +433,12 @@ impl<'lock> Environment<'lock> {
             .map(|pkgs| pkgs.filter_map(LockedPackageRef::as_pypi))
     }
 
+    /// Returns whether this environment has any pypi packages for the specified platform.
+    pub fn has_pypi_packages(&self, platform: Platform) -> bool {
+        self.pypi_packages(platform)
+            .is_some_and(|mut packages| packages.next().is_some())
+    }
+
     /// Creates a [`OwnedEnvironment`] from this environment.
     pub fn to_owned(self) -> OwnedEnvironment {
         OwnedEnvironment {
@@ -516,7 +536,7 @@ mod test {
         str::FromStr,
     };
 
-    use rattler_conda_types::Platform;
+    use rattler_conda_types::{Platform, RepoDataRecord};
     use rstest::*;
 
     use super::{LockFile, DEFAULT_ENVIRONMENT_NAME};
@@ -536,6 +556,8 @@ mod test {
     #[case::v5_with_and_without_purl("v5/similar-with-and-without-purl.yml")]
     #[case::v6_conda_source_path("v6/conda-path-lock.yml")]
     #[case::v6_derived_channel("v6/derived-channel-lock.yml")]
+    #[case::v6_sources("v6/sources-lock.yml")]
+    #[case::v6_options("v6/options-lock.yml")]
     fn test_parse(#[case] file_name: &str) {
         let path = Path::new(env!("CARGO_MANIFEST_DIR"))
             .join("../../test-data/conda-lock")
@@ -600,4 +622,104 @@ mod test {
             .map(|p| p.location().to_string())
             .collect::<Vec<_>>());
     }
+
+    #[test]
+    fn test_has_pypi_packages() {
+        // v4
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/conda-lock")
+            .join("v4/pypi-matplotlib-lock.yml");
+        let conda_lock = LockFile::from_path(&path).unwrap();
+
+        assert!(conda_lock
+            .environment(DEFAULT_ENVIRONMENT_NAME)
+            .unwrap()
+            .has_pypi_packages(Platform::Linux64));
+
+        // v6
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/conda-lock")
+            .join("v6/numpy-as-pypi-lock.yml");
+        let conda_lock = LockFile::from_path(&path).unwrap();
+
+        assert!(conda_lock
+            .environment(DEFAULT_ENVIRONMENT_NAME)
+            .unwrap()
+            .has_pypi_packages(Platform::OsxArm64));
+
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/conda-lock")
+            .join("v6/python-from-conda-only-lock.yml");
+        let conda_lock = LockFile::from_path(&path).unwrap();
+
+        assert!(!conda_lock
+            .environment(DEFAULT_ENVIRONMENT_NAME)
+            .unwrap()
+            .has_pypi_packages(Platform::OsxArm64));
+    }
+
+    #[test]
+    fn test_is_empty() {
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/conda-lock")
+            .join("v6/empty-lock.yml");
+        let conda_lock = LockFile::from_path(&path).unwrap();
+        assert!(conda_lock.is_empty());
+
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/conda-lock")
+            .join("v6/python-from-conda-only-lock.yml");
+        let conda_lock = LockFile::from_path(&path).unwrap();
+        assert!(!conda_lock.is_empty());
+    }
+
+    #[test]
+    fn solve_roundtrip() {
+        // load repodata from JSON
+        let path = Path::new(env!("CARGO_MANIFEST_DIR"))
+            .join("../../test-data/repodata-records/_libgcc_mutex-0.1-conda_forge.json");
+        let content = std::fs::read_to_string(&path).unwrap();
+        let repodata_record: RepoDataRecord = serde_json::from_str(&content).unwrap();
+
+        // check that the repodata record is as expected
+        assert_eq!(repodata_record.package_record.arch, None);
+        assert_eq!(repodata_record.package_record.platform, None);
+
+        // create a lockfile with the repodata record
+        let lock_file = LockFile::builder()
+            .with_conda_package(
+                DEFAULT_ENVIRONMENT_NAME,
+                Platform::Linux64,
+                repodata_record.clone().into(),
+            )
+            .finish();
+
+        // serialize the lockfile
+        let rendered_lock_file = lock_file.render_to_string().unwrap();
+
+        // parse the lockfile
+        let parsed_lock_file = LockFile::from_str(&rendered_lock_file).unwrap();
+        // get repodata record from parsed lockfile
+        let repodata_records = parsed_lock_file
+            .environment(DEFAULT_ENVIRONMENT_NAME)
+            .unwrap()
+            .conda_repodata_records(Platform::Linux64)
+            .unwrap()
+            .unwrap();
+
+        // These are not equal because the one from `repodata_records[0]` contains arch and platform.
+        let repodata_record_two = repodata_records[0].clone();
+        assert_eq!(
+            repodata_record_two.package_record.arch,
+            Some("x86_64".to_string())
+        );
+        assert_eq!(
+            repodata_record_two.package_record.platform,
+            Some("linux".to_string())
+        );
+
+        // But if we render it again, the lockfile should look the same at least
+        let rerendered_lock_file_two = parsed_lock_file.render_to_string().unwrap();
+        assert_eq!(rendered_lock_file, rerendered_lock_file_two);
+    }
 }
