@@ -13,14 +13,15 @@ use chrono::{DateTime, Utc};
 use conda_sorting::SolvableSorter;
 use itertools::Itertools;
 use rattler_conda_types::{
-    package::ArchiveType, GenericVirtualPackage, MatchSpec, Matches, NamelessMatchSpec,
-    PackageName, PackageRecord, ParseMatchSpecError, ParseStrictness, RepoDataRecord,
+    package::ArchiveType, version_spec::EqualityOperator, BuildNumberSpec, GenericVirtualPackage,
+    MatchSpec, Matches, NamelessMatchSpec, OrdOperator, PackageName, PackageRecord,
+    ParseMatchSpecError, ParseStrictness, RepoDataRecord, SolverResult, StringMatcher, VersionSpec,
 };
 use resolvo::{
     utils::{Pool, VersionSet},
-    Candidates, Dependencies, DependencyProvider, Interner, KnownDependencies, NameId, Problem,
-    Requirement, SolvableId, Solver as LibSolvRsSolver, SolverCache, StringId,
-    UnsolvableOrCancelled, VersionSetId, VersionSetUnionId,
+    Candidates, Dependencies, DependencyProvider, HintDependenciesAvailable, Interner,
+    KnownDependencies, NameId, Problem, Requirement, SolvableId, Solver as LibSolvRsSolver,
+    SolverCache, StringId, UnsolvableOrCancelled, VersionSetId, VersionSetUnionId,
 };
 
 use crate::{
@@ -49,29 +50,48 @@ impl<'a> FromIterator<&'a RepoDataRecord> for RepoData<'a> {
 impl<'a> SolverRepoData<'a> for RepoData<'a> {}
 
 /// Wrapper around `MatchSpec` so that we can use it in the `resolvo` pool
-#[repr(transparent)]
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct SolverMatchSpec<'a> {
     inner: NamelessMatchSpec,
+    feature: Option<String>,
     _marker: PhantomData<&'a PackageRecord>,
 }
 
-impl<'a> From<NamelessMatchSpec> for SolverMatchSpec<'a> {
+impl<'a> SolverMatchSpec<'a> {
+    /// Returns a reference to this match spec with the given feature enabled
+    pub fn with_feature(&self, feature: String) -> SolverMatchSpec<'a> {
+        Self {
+            inner: self.inner.clone(),
+            feature: Some(feature),
+            _marker: self._marker,
+        }
+    }
+
+    /// Returns a mutable reference to this match spec after enabling the given
+    /// feature
+    pub fn set_feature(&mut self, feature: String) -> &SolverMatchSpec<'a> {
+        self.feature = Some(feature);
+        self
+    }
+}
+
+impl From<NamelessMatchSpec> for SolverMatchSpec<'_> {
     fn from(value: NamelessMatchSpec) -> Self {
         Self {
             inner: value,
+            feature: None,
             _marker: PhantomData,
         }
     }
 }
 
-impl<'a> Display for SolverMatchSpec<'a> {
+impl Display for SolverMatchSpec<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.inner)
     }
 }
 
-impl<'a> Deref for SolverMatchSpec<'a> {
+impl Deref for SolverMatchSpec<'_> {
     type Target = NamelessMatchSpec;
 
     fn deref(&self) -> &Self::Target {
@@ -89,17 +109,20 @@ pub enum SolverPackageRecord<'a> {
     /// Represents a record from the repodata
     Record(&'a RepoDataRecord),
 
+    /// Represents a record with a specific feature enabled
+    RecordWithFeature(&'a RepoDataRecord, String),
+
     /// Represents a virtual package.
     VirtualPackage(&'a GenericVirtualPackage),
 }
 
-impl<'a> PartialOrd<Self> for SolverPackageRecord<'a> {
+impl PartialOrd<Self> for SolverPackageRecord<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
     }
 }
 
-impl<'a> Ord for SolverPackageRecord<'a> {
+impl Ord for SolverPackageRecord<'_> {
     fn cmp(&self, other: &Self) -> Ordering {
         self.name()
             .cmp(other.name())
@@ -109,17 +132,21 @@ impl<'a> Ord for SolverPackageRecord<'a> {
     }
 }
 
-impl<'a> SolverPackageRecord<'a> {
+impl SolverPackageRecord<'_> {
     fn name(&self) -> &PackageName {
         match self {
-            SolverPackageRecord::Record(rec) => &rec.package_record.name,
+            SolverPackageRecord::Record(rec) | SolverPackageRecord::RecordWithFeature(rec, _) => {
+                &rec.package_record.name
+            }
             SolverPackageRecord::VirtualPackage(rec) => &rec.name,
         }
     }
 
     fn version(&self) -> &rattler_conda_types::Version {
         match self {
-            SolverPackageRecord::Record(rec) => rec.package_record.version.version(),
+            SolverPackageRecord::Record(rec) | SolverPackageRecord::RecordWithFeature(rec, _) => {
+                rec.package_record.version.version()
+            }
             SolverPackageRecord::VirtualPackage(rec) => &rec.version,
         }
     }
@@ -127,32 +154,41 @@ impl<'a> SolverPackageRecord<'a> {
     fn track_features(&self) -> &[String] {
         const EMPTY: [String; 0] = [];
         match self {
-            SolverPackageRecord::Record(rec) => &rec.package_record.track_features,
+            SolverPackageRecord::Record(rec) | SolverPackageRecord::RecordWithFeature(rec, _) => {
+                &rec.package_record.track_features
+            }
             SolverPackageRecord::VirtualPackage(_rec) => &EMPTY,
         }
     }
 
     fn build_number(&self) -> u64 {
         match self {
-            SolverPackageRecord::Record(rec) => rec.package_record.build_number,
+            SolverPackageRecord::Record(rec) | SolverPackageRecord::RecordWithFeature(rec, _) => {
+                rec.package_record.build_number
+            }
             SolverPackageRecord::VirtualPackage(_rec) => 0,
         }
     }
 
     fn timestamp(&self) -> Option<&chrono::DateTime<chrono::Utc>> {
         match self {
-            SolverPackageRecord::Record(rec) => rec.package_record.timestamp.as_ref(),
+            SolverPackageRecord::Record(rec) | SolverPackageRecord::RecordWithFeature(rec, _) => {
+                rec.package_record.timestamp.as_ref()
+            }
             SolverPackageRecord::VirtualPackage(_rec) => None,
         }
     }
 }
 
-impl<'a> Display for SolverPackageRecord<'a> {
+impl Display for SolverPackageRecord<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
             SolverPackageRecord::Record(rec) => {
                 write!(f, "{}", &rec.package_record)
             }
+            SolverPackageRecord::RecordWithFeature(rec, feature) => {
+                write!(f, "{}[{}]", &rec.package_record, feature)
+            }
             SolverPackageRecord::VirtualPackage(rec) => {
                 write!(f, "{rec}")
             }
@@ -160,20 +196,66 @@ impl<'a> Display for SolverPackageRecord<'a> {
     }
 }
 
+/// Represents the type of name that is being used in the pool.
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+pub enum NameType {
+    /// A package name without a feature.
+    Base(String),
+
+    /// A package name with a feature.
+    BaseWithFeature(String, String),
+}
+
+impl Display for NameType {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            NameType::Base(name) => write!(f, "{name}"),
+            NameType::BaseWithFeature(name, feature) => write!(f, "{name}[{feature}]"),
+        }
+    }
+}
+
+impl Ord for NameType {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        match (self, other) {
+            // Compare names first
+            (NameType::Base(name1), NameType::Base(name2))
+            | (NameType::BaseWithFeature(name1, _), NameType::BaseWithFeature(name2, _)) => {
+                name1.cmp(name2)
+            }
+            // WithoutFeature comes before WithFeature
+            (NameType::Base(_), NameType::BaseWithFeature(_, _)) => std::cmp::Ordering::Greater,
+            (NameType::BaseWithFeature(_, _), NameType::Base(_)) => std::cmp::Ordering::Less,
+        }
+    }
+}
+
+impl PartialOrd for NameType {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl From<&str> for NameType {
+    fn from(value: &str) -> Self {
+        NameType::Base(value.to_owned())
+    }
+}
+
 /// An implement of [`resolvo::DependencyProvider`] that implements the
 /// ecosystem behavior for conda. This allows resolvo to solve for conda
 /// packages.
 #[derive(Default)]
 pub struct CondaDependencyProvider<'a> {
     /// The pool that deduplicates data used by the provider.
-    pub pool: Pool<SolverMatchSpec<'a>, String>,
+    pub pool: Pool<SolverMatchSpec<'a>, NameType>,
 
     records: HashMap<NameId, Candidates>,
 
     matchspec_to_highest_version:
         RefCell<HashMap<VersionSetId, Option<(rattler_conda_types::Version, bool)>>>,
 
-    parse_match_spec_cache: RefCell<HashMap<&'a str, VersionSetId>>,
+    parse_match_spec_cache: RefCell<HashMap<String, Vec<VersionSetId>>>,
 
     stop_time: Option<std::time::SystemTime>,
 
@@ -297,94 +379,121 @@ impl<'a> CondaDependencyProvider<'a> {
                     pool.intern_package_name(record.package_record.name.as_normalized());
                 let solvable_id =
                     pool.intern_solvable(package_name, SolverPackageRecord::Record(record));
-                let candidates = records.entry(package_name).or_default();
-                candidates.candidates.push(solvable_id);
 
-                // Filter out any records that are newer than a specific date.
-                match (&exclude_newer, &record.package_record.timestamp) {
-                    (Some(exclude_newer), Some(record_timestamp))
-                        if record_timestamp > exclude_newer =>
-                    {
-                        let reason = pool.intern_string(format!(
-                            "the package is uploaded after the cutoff date of {exclude_newer}"
+                // Collect all candidates first
+                let mut all_entries = vec![(package_name, solvable_id)];
+
+                // Add feature-enabled solvables
+                for feature in record.package_record.extra_depends.keys() {
+                    let package_name_with_feature =
+                        pool.intern_package_name(NameType::BaseWithFeature(
+                            record.package_record.name.as_normalized().to_owned(),
+                            feature.clone(),
                         ));
-                        candidates.excluded.push((solvable_id, reason));
-                    }
-                    _ => {}
+                    let feature_solvable = pool.intern_solvable(
+                        package_name_with_feature,
+                        SolverPackageRecord::RecordWithFeature(record, feature.clone()),
+                    );
+                    let package_name_with_feature =
+                        pool.intern_package_name(NameType::BaseWithFeature(
+                            record.package_record.name.as_normalized().to_owned(),
+                            feature.clone(),
+                        ));
+
+                    all_entries.push((package_name_with_feature, feature_solvable));
                 }
 
-                // Add to excluded when package is not in the specified channel.
-                if !channel_specific_specs.is_empty() {
-                    if let Some(spec) = channel_specific_specs.iter().find(|&&spec| {
-                        spec.name
-                            .as_ref()
-                            .expect("expecting a name")
-                            .as_normalized()
-                            == record.package_record.name.as_normalized()
-                    }) {
-                        // Check if the spec has a channel, and compare it to the repodata channel
-                        if let Some(spec_channel) = &spec.channel {
-                            if record.channel.as_ref() != Some(&spec_channel.canonical_name()) {
-                                tracing::debug!("Ignoring {} {} because it was not requested from that channel.", &record.package_record.name.as_normalized(), match &record.channel {
-                                    Some(channel) => format!("from {}", &channel),
-                                    None => "without a channel".to_string(),
-                                });
-                                // Add record to the excluded with reason of being in the non
-                                // requested channel.
-                                let message = format!(
-                                    "candidate not in requested channel: '{}'",
-                                    spec_channel
-                                        .name
-                                        .clone()
-                                        .unwrap_or(spec_channel.base_url.to_string())
-                                );
-                                candidates
-                                    .excluded
-                                    .push((solvable_id, pool.intern_string(message)));
-                                continue;
+                // Update records with all entries in a single mutable borrow
+                for (pkg_name, solvable) in all_entries {
+                    let candidates = records.entry(pkg_name).or_default();
+                    candidates.candidates.push(solvable);
+
+                    // Filter out any records that are newer than a specific date.
+                    match (&exclude_newer, &record.package_record.timestamp) {
+                        (Some(exclude_newer), Some(record_timestamp))
+                            if record_timestamp > exclude_newer =>
+                        {
+                            let reason = pool.intern_string(format!(
+                                "the package is uploaded after the cutoff date of {exclude_newer}"
+                            ));
+                            candidates.excluded.push((solvable, reason));
+                        }
+                        _ => {}
+                    }
+
+                    // Add to excluded when package is not in the specified channel.
+                    if !channel_specific_specs.is_empty() {
+                        if let Some(spec) = channel_specific_specs.iter().find(|&&spec| {
+                            spec.name
+                                .as_ref()
+                                .expect("expecting a name")
+                                .as_normalized()
+                                == record.package_record.name.as_normalized()
+                        }) {
+                            // Check if the spec has a channel, and compare it to the repodata
+                            // channel
+                            if let Some(spec_channel) = &spec.channel {
+                                if record.channel.as_ref() != Some(&spec_channel.canonical_name()) {
+                                    tracing::debug!("Ignoring {} {} because it was not requested from that channel.", &record.package_record.name.as_normalized(), match &record.channel {
+                                        Some(channel) => format!("from {}", &channel),
+                                        None => "without a channel".to_string(),
+                                    });
+                                    // Add record to the excluded with reason of being in the non
+                                    // requested channel.
+                                    let message = format!(
+                                        "candidate not in requested channel: '{}'",
+                                        spec_channel
+                                            .name
+                                            .clone()
+                                            .unwrap_or(spec_channel.base_url.to_string())
+                                    );
+                                    candidates
+                                        .excluded
+                                        .push((solvable, pool.intern_string(message)));
+                                    continue;
+                                }
                             }
                         }
                     }
-                }
 
-                // Enforce channel priority
-                // This function makes the assumption that the records are given in order of the
-                // channels.
-                if let (Some(first_channel), ChannelPriority::Strict) = (
-                    package_name_found_in_channel.get(record.package_record.name.as_normalized()),
-                    channel_priority,
-                ) {
-                    // Add the record to the excluded list when it is from a different channel.
-                    if first_channel != &&record.channel {
-                        if let Some(channel) = &record.channel {
-                            tracing::debug!(
-                                "Ignoring '{}' from '{}' because of strict channel priority.",
-                                &record.package_record.name.as_normalized(),
-                                channel
-                            );
-                            candidates.excluded.push((
-                                solvable_id,
-                                pool.intern_string(format!(
-                                    "due to strict channel priority not using this option from: '{channel}'",
-                                )),
-                            ));
-                        } else {
-                            tracing::debug!(
-                                "Ignoring '{}' without a channel because of strict channel priority.",
-                                &record.package_record.name.as_normalized(),
-                            );
-                            candidates.excluded.push((
-                                solvable_id,
-                                pool.intern_string("due to strict channel priority not using from an unknown channel".to_string()),
-                            ));
+                    // Enforce channel priority
+                    if let (Some(first_channel), ChannelPriority::Strict) = (
+                        package_name_found_in_channel
+                            .get(record.package_record.name.as_normalized()),
+                        channel_priority,
+                    ) {
+                        // Add the record to the excluded list when it is from a different channel.
+                        if first_channel != &&record.channel {
+                            if let Some(channel) = &record.channel {
+                                tracing::debug!(
+                                    "Ignoring '{}' from '{}' because of strict channel priority.",
+                                    &record.package_record.name.as_normalized(),
+                                    channel
+                                );
+                                candidates.excluded.push((
+                                    solvable,
+                                    pool.intern_string(format!(
+                                        "due to strict channel priority not using this option from: '{channel}'",
+                                    )),
+                                ));
+                            } else {
+                                tracing::debug!(
+                                    "Ignoring '{}' without a channel because of strict channel priority.",
+                                    &record.package_record.name.as_normalized(),
+                                );
+                                candidates.excluded.push((
+                                    solvable,
+                                    pool.intern_string("due to strict channel priority not using from an unknown channel".to_string()),
+                                ));
+                            }
+                            continue;
                         }
-                        continue;
+                    } else {
+                        package_name_found_in_channel.insert(
+                            record.package_record.name.as_normalized().to_string(),
+                            &record.channel,
+                        );
                     }
-                } else {
-                    package_name_found_in_channel.insert(
-                        record.package_record.name.as_normalized().to_string(),
-                        &record.channel,
-                    );
                 }
             }
         }
@@ -408,7 +517,7 @@ impl<'a> CondaDependencyProvider<'a> {
 
         // The dependencies for all candidates are always available.
         for candidates in records.values_mut() {
-            candidates.hint_dependencies_available = candidates.candidates.clone();
+            candidates.hint_dependencies_available = HintDependenciesAvailable::All;
         }
 
         Ok(Self {
@@ -434,7 +543,7 @@ pub enum CancelReason {
     Timeout,
 }
 
-impl<'a> Interner for CondaDependencyProvider<'a> {
+impl Interner for CondaDependencyProvider<'_> {
     fn display_solvable(&self, solvable: SolvableId) -> impl Display + '_ {
         &self.pool.resolve_solvable(solvable).record
     }
@@ -482,7 +591,7 @@ impl<'a> Interner for CondaDependencyProvider<'a> {
     }
 }
 
-impl<'a> DependencyProvider for CondaDependencyProvider<'a> {
+impl DependencyProvider for CondaDependencyProvider<'_> {
     async fn sort_candidates(&self, solver: &SolverCache<Self>, solvables: &mut [SolvableId]) {
         if solvables.is_empty() {
             // Short circuit if there are no solvables to sort
@@ -522,40 +631,104 @@ impl<'a> DependencyProvider for CondaDependencyProvider<'a> {
 
     async fn get_dependencies(&self, solvable: SolvableId) -> Dependencies {
         let mut dependencies = KnownDependencies::default();
-        let SolverPackageRecord::Record(rec) = self.pool.resolve_solvable(solvable).record else {
-            return Dependencies::Known(dependencies);
+
+        // Get the record and any feature that might be enabled
+        let (record, feature) = match &self.pool.resolve_solvable(solvable).record {
+            SolverPackageRecord::Record(rec) => (rec, None),
+            SolverPackageRecord::RecordWithFeature(rec, feature) => (rec, Some(feature)),
+            SolverPackageRecord::VirtualPackage(_) => return Dependencies::Known(dependencies),
         };
 
         let mut parse_match_spec_cache = self.parse_match_spec_cache.borrow_mut();
-        for depends in rec.package_record.depends.iter() {
-            let version_set_id =
-                match parse_match_spec(&self.pool, depends, &mut parse_match_spec_cache) {
-                    Ok(version_set_id) => version_set_id,
-                    Err(e) => {
-                        let reason = self.pool.intern_string(format!(
-                            "the dependency '{depends}' failed to parse: {e}",
-                        ));
 
-                        return Dependencies::Unknown(reason);
-                    }
+        // If this is a feature-enabled package, add its feature dependencies
+        if let Some(feature_name) = feature {
+            // Find the feature's dependencies
+            if let Some(deps) = record.package_record.extra_depends.get(feature_name) {
+                // Add each dependency for this feature
+                for req in deps {
+                    let version_set_id = match parse_match_spec(
+                        &self.pool,
+                        req,
+                        &mut parse_match_spec_cache,
+                    ) {
+                        Ok(version_set_id) => version_set_id,
+                        Err(e) => {
+                            let reason = self.pool.intern_string(format!(
+                                "the optional dependency '{req}' for feature '{feature_name}' failed to parse: {e}"
+                            ));
+                            return Dependencies::Unknown(reason);
+                        }
+                    };
+
+                    dependencies
+                        .requirements
+                        .extend(version_set_id.into_iter().map(Requirement::from));
+                }
+
+                // Add a dependency back to the base package with exact version
+                let base_spec = MatchSpec {
+                    name: Some(record.package_record.name.clone()),
+                    version: Some(VersionSpec::Exact(
+                        EqualityOperator::Equals,
+                        record.package_record.version.version().clone(),
+                    )),
+                    build: Some(StringMatcher::Exact(record.package_record.build.clone())),
+                    build_number: Some(BuildNumberSpec::new(
+                        OrdOperator::Eq,
+                        record.package_record.build_number,
+                    )),
+                    subdir: Some(record.package_record.subdir.clone()),
+                    md5: record.package_record.md5,
+                    sha256: record.package_record.sha256,
+                    extras: None,
+                    ..Default::default()
                 };
 
-            dependencies.requirements.push(version_set_id.into());
-        }
+                let (name, nameless_spec) = base_spec.into_nameless();
+                let name_id = self.pool.intern_package_name(
+                    name.expect("cannot use matchspec without a name")
+                        .as_normalized(),
+                );
+                let version_set_id = self.pool.intern_version_set(name_id, nameless_spec.into());
+                dependencies.requirements.push(version_set_id.into());
+            }
+        } else {
+            // Add regular dependencies
+            for depends in record.package_record.depends.iter() {
+                let version_set_id =
+                    match parse_match_spec(&self.pool, depends, &mut parse_match_spec_cache) {
+                        Ok(version_set_id) => version_set_id,
+                        Err(e) => {
+                            let reason = self.pool.intern_string(format!(
+                                "the dependency '{depends}' failed to parse: {e}",
+                            ));
 
-        for constrains in rec.package_record.constrains.iter() {
-            let version_set_id =
-                match parse_match_spec(&self.pool, constrains, &mut parse_match_spec_cache) {
-                    Ok(version_set_id) => version_set_id,
-                    Err(e) => {
-                        let reason = self.pool.intern_string(format!(
-                            "the constrains '{constrains}' failed to parse: {e}",
-                        ));
+                            return Dependencies::Unknown(reason);
+                        }
+                    };
 
-                        return Dependencies::Unknown(reason);
-                    }
-                };
-            dependencies.constrains.push(version_set_id);
+                dependencies
+                    .requirements
+                    .extend(version_set_id.into_iter().map(Requirement::from));
+            }
+
+            for constrains in record.package_record.constrains.iter() {
+                let version_set_id =
+                    match parse_match_spec(&self.pool, constrains, &mut parse_match_spec_cache) {
+                        Ok(version_set_id) => version_set_id,
+                        Err(e) => {
+                            let reason = self.pool.intern_string(format!(
+                                "the constrains '{constrains}' failed to parse: {e}",
+                            ));
+
+                            return Dependencies::Unknown(reason);
+                        }
+                    };
+                for version_set_id in version_set_id {
+                    dependencies.constrains.push(version_set_id);
+                }
+            }
         }
 
         Dependencies::Known(dependencies)
@@ -575,7 +748,24 @@ impl<'a> DependencyProvider for CondaDependencyProvider<'a> {
             .filter(|c| {
                 let record = &self.pool.resolve_solvable(*c).record;
                 match record {
-                    SolverPackageRecord::Record(rec) => spec.matches(*rec) != inverse,
+                    SolverPackageRecord::Record(rec) => {
+                        // Base package matches if spec matches and no features are required
+
+                        spec.matches(*rec) != inverse
+                    }
+                    SolverPackageRecord::RecordWithFeature(rec, feature) => {
+                        // Feature-enabled package matches if spec matches and feature is required
+
+                        if spec.matches(*rec) {
+                            if let Some(spec_feature) = &spec.feature {
+                                (*spec_feature == *feature) != inverse
+                            } else {
+                                inverse
+                            }
+                        } else {
+                            inverse
+                        }
+                    }
                     SolverPackageRecord::VirtualPackage(GenericVirtualPackage {
                         version,
                         build_string,
@@ -625,7 +815,7 @@ impl super::SolverImpl for Solver {
     >(
         &mut self,
         task: SolverTask<TAvailablePackagesIterator>,
-    ) -> Result<Vec<RepoDataRecord>, SolveError> {
+    ) -> Result<SolverResult, SolveError> {
         let stop_time = task
             .timeout
             .map(|timeout| std::time::SystemTime::now() + timeout);
@@ -651,16 +841,39 @@ impl super::SolverImpl for Solver {
                 .intern_version_set(name_id, NamelessMatchSpec::default().into())
         });
 
-        let root_requirements = task.specs.iter().map(|spec| {
+        let root_requirements = task.specs.iter().flat_map(|spec| {
             let (name, nameless_spec) = spec.clone().into_nameless();
+            let features = &spec.extras;
             let name = name.expect("cannot use matchspec without a name");
             let name_id = provider.pool.intern_package_name(name.as_normalized());
-            provider
+            let mut reqs = vec![provider
                 .pool
-                .intern_version_set(name_id, nameless_spec.into())
+                .intern_version_set(name_id, nameless_spec.clone().into())];
+
+            // Add requirements for optional features if specified
+            if let Some(features) = features {
+                for feature in features {
+                    // Create a version set that matches the feature-enabled package
+                    let package_name_with_feature = NameType::BaseWithFeature(
+                        name.as_normalized().to_owned(),
+                        feature.to_string(),
+                    );
+                    let feature_name_id =
+                        provider.pool.intern_package_name(package_name_with_feature);
+
+                    let mut solver_match_spec: SolverMatchSpec<'_> = nameless_spec.clone().into();
+                    let _ = solver_match_spec.set_feature(feature.to_string());
+
+                    let feature_version_set = provider
+                        .pool
+                        .intern_version_set(feature_name_id, solver_match_spec);
+                    reqs.push(feature_version_set);
+                }
+            }
+            reqs
         });
 
-        let all_requirements = virtual_package_requirements
+        let all_requirements: Vec<Requirement> = virtual_package_requirements
             .chain(root_requirements)
             .map(Requirement::from)
             .collect();
@@ -677,7 +890,7 @@ impl super::SolverImpl for Solver {
             .collect();
 
         let problem = Problem::new()
-            .requirements(all_requirements)
+            .requirements(all_requirements.clone())
             .constraints(root_constraints);
 
         // Construct a solver and solve the problems in the queue
@@ -694,37 +907,70 @@ impl super::SolverImpl for Solver {
         })?;
 
         // Get the resulting packages from the solver.
-        let required_records = solvables
-            .into_iter()
-            .filter_map(
-                |id| match solver.provider().pool.resolve_solvable(id).record {
-                    SolverPackageRecord::Record(rec) => Some(rec.clone()),
-                    SolverPackageRecord::VirtualPackage(_) => None,
-                },
-            )
-            .collect();
+        let mut features: HashMap<PackageName, Vec<String>> = HashMap::new();
+        let mut records = Vec::new();
 
-        Ok(required_records)
+        for id in solvables {
+            match &solver.provider().pool.resolve_solvable(id).record {
+                SolverPackageRecord::Record(rec) => {
+                    records.push((*rec).clone());
+                }
+                SolverPackageRecord::RecordWithFeature(rec, feature) => {
+                    features
+                        .entry(rec.package_record.name.clone())
+                        .or_default()
+                        .push(feature.clone());
+                }
+                SolverPackageRecord::VirtualPackage(_) => {}
+            }
+        }
+
+        Ok(SolverResult { records, features })
     }
 }
 
-fn parse_match_spec<'a>(
-    pool: &Pool<SolverMatchSpec<'a>>,
-    spec_str: &'a str,
-    parse_match_spec_cache: &mut HashMap<&'a str, VersionSetId>,
-) -> Result<VersionSetId, ParseMatchSpecError> {
+fn parse_match_spec(
+    pool: &Pool<SolverMatchSpec<'_>, NameType>,
+    spec_str: &str,
+    parse_match_spec_cache: &mut HashMap<String, Vec<VersionSetId>>,
+) -> Result<Vec<VersionSetId>, ParseMatchSpecError> {
     if let Some(spec_id) = parse_match_spec_cache.get(spec_str) {
-        Ok(*spec_id)
+        return Ok(spec_id.clone());
+    }
+
+    let match_spec = MatchSpec::from_str(spec_str, ParseStrictness::Lenient)?;
+    let (name, spec) = match_spec.into_nameless();
+
+    let mut version_set_ids = vec![];
+    if let Some(ref features) = spec.extras {
+        let spec_with_feature: SolverMatchSpec<'_> = spec.clone().into();
+
+        for feature in features {
+            let name_with_feature = NameType::BaseWithFeature(
+                name.as_ref()
+                    .expect("Packages with no name are not supported")
+                    .as_normalized()
+                    .to_owned(),
+                feature.to_string(),
+            );
+            let dependency_name = pool.intern_package_name(name_with_feature);
+
+            let version_set_id = pool.intern_version_set(
+                dependency_name,
+                spec_with_feature.with_feature(feature.to_string()),
+            );
+            version_set_ids.push(version_set_id);
+        }
     } else {
-        let match_spec = MatchSpec::from_str(spec_str, ParseStrictness::Lenient)?;
-        let (name, spec) = match_spec.into_nameless();
         let dependency_name = pool.intern_package_name(
             name.as_ref()
-                .expect("match specs without names are not supported")
+                .expect("Packages with no name are not supported")
                 .as_normalized(),
         );
         let version_set_id = pool.intern_version_set(dependency_name, spec.into());
-        parse_match_spec_cache.insert(spec_str, version_set_id);
-        Ok(version_set_id)
+        version_set_ids.push(version_set_id);
     }
+    parse_match_spec_cache.insert(spec_str.to_string(), version_set_ids.clone());
+
+    Ok(version_set_ids)
 }
