@@ -13,6 +13,7 @@ use crate::prelude::*;
 ///
 /// * [`DeleteFile`](crate::DeleteFile)
 /// * [`MoveFile`](crate::MoveFile)
+/// * [`MoveFileEx`](crate::MoveFileEx)
 /// * [`ReplaceFile`](crate::ReplaceFile)
 pub fn CopyFile(
 	existing_file: &str,
@@ -55,6 +56,7 @@ pub fn CreateDirectory(
 ///
 /// * [`CopyFile`](crate::CopyFile)
 /// * [`MoveFile`](crate::MoveFile)
+/// * [`MoveFileEx`](crate::MoveFileEx)
 /// * [`ReplaceFile`](crate::ReplaceFile)
 pub fn DeleteFile(file_name: &str) -> SysResult<()> {
 	bool_to_sysresult(
@@ -92,29 +94,37 @@ pub fn ExitThread(exit_code: u32) {
 #[must_use]
 pub fn ExpandEnvironmentStrings(src: &str) -> SysResult<String> {
 	let wsrc = WString::from_str(src);
-	let len = unsafe {
-		ffi::ExpandEnvironmentStringsW(
-			wsrc.as_ptr(),
-			std::ptr::null_mut(),
-			0,
-		)
-	};
+	let mut buf_sz = match unsafe {
+		ffi::ExpandEnvironmentStringsW(wsrc.as_ptr(), std::ptr::null_mut(), 0)
+	} {
+		0 => return Err(GetLastError()),
+		n => n,
+	}; // includes terminating null count
 
-	let mut buf = WString::new_alloc_buf(len as _);
-	bool_to_sysresult(
-		unsafe {
-			ffi::ExpandEnvironmentStringsW(
-				wsrc.as_ptr(),
-				buf.as_mut_ptr(),
-				len,
-			)
-		} as _,
-	).map(|_| buf.to_string())
+	loop {
+		let mut buf = WString::new_alloc_buf(buf_sz as _);
+		let required_sz = match unsafe {
+			ffi::ExpandEnvironmentStringsW(wsrc.as_ptr(), buf.as_mut_ptr(), buf_sz)
+		} {
+			0 => return Err(GetLastError()),
+			n => n,
+		}; // plus terminating null count
+
+		if required_sz <= buf_sz {
+			return Ok(buf.to_string());
+		}
+
+		buf_sz = required_sz; // includes terminating null count; set the new buffer size to try again
+	}
 }
 
 /// [`FileTimeToSystemTime`](https://learn.microsoft.com/en-us/windows/win32/api/timezoneapi/nf-timezoneapi-filetimetosystemtime)
 /// function.
 ///
+/// Note that the system time is UTC. In order to convert to local time, you
+/// must also pass the returned `SYSTEMTIME` to
+/// [`SystemTimeToTzSpecificLocalTime`](crate::SystemTimeToTzSpecificLocalTime).
+///
 /// # Related functions
 ///
 /// * [`GetLocalTime`](crate::GetLocalTime)
@@ -214,15 +224,28 @@ pub fn GetComputerName() -> SysResult<String> {
 /// function.
 #[must_use]
 pub fn GetCurrentDirectory() -> SysResult<String> {
-	let mut buf = WString::new_alloc_buf(MAX_PATH + 1);
-	bool_to_sysresult(
-		unsafe {
-			ffi::GetCurrentDirectoryW(
-				buf.buf_len() as _,
-				buf.as_mut_ptr(),
-			)
-		} as _,
-	).map(|_| buf.to_string())
+	let mut buf_sz = match unsafe {
+		ffi::GetCurrentDirectoryW(0, std::ptr::null_mut())
+	} {
+		0 => return Err(GetLastError()),
+		n => n,
+	}; // includes terminating null count
+
+	loop {
+		let mut buf = WString::new_alloc_buf(buf_sz as _);
+		let returned_chars = match unsafe {
+			ffi::GetCurrentDirectoryW(buf_sz, buf.as_mut_ptr())
+		} {
+			0 => return Err(GetLastError()),
+			n => n,
+		};
+
+		if returned_chars < buf_sz {
+			return Ok(buf.to_string());
+		}
+
+		buf_sz = returned_chars; // includes terminating null count; set the new buffer size to try again
+	}
 }
 
 /// [`GetCurrentProcessId`](https://learn.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getcurrentprocessid)
@@ -317,7 +340,9 @@ pub fn GetDiskSpaceInformation(
 pub fn GetEnvironmentStrings() -> SysResult<Vec<(String, String)>> {
 	ptr_to_sysresult(unsafe { ffi::GetEnvironmentStringsW() } as _)
 		.map(|ptr| {
-			let vec_entries = parse_multi_z_str(ptr as *mut _ as _);
+			let vec_entries = unsafe {
+				parse_multi_z_str(ptr as *mut _ as _, None)
+			};
 			unsafe { ffi::FreeEnvironmentStringsW(ptr); }
 			vec_entries.iter()
 				.map(|env_str| {
@@ -330,6 +355,62 @@ pub fn GetEnvironmentStrings() -> SysResult<Vec<(String, String)>> {
 		})
 }
 
+/// [`GetFileAttributes`](https://learn.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getfileattributesw)
+/// function.
+///
+/// # Examples
+///
+/// Checking whether a file or folder exists:
+///
+/// ```no_run
+/// use winsafe::{self as w, prelude::*};
+///
+/// let file_exists = w::GetFileAttributes("C:\\Temp\\test.txt").is_ok();
+/// ```
+///
+/// Retrieving various information about a file or folder path:
+///
+/// ```no_run
+/// use winsafe::{self as w, prelude::*, co};
+///
+/// let flags = w::GetFileAttributes("C:\\Temp\\test.txt")?;
+///
+/// let is_compressed = flags.has(co::FILE_ATTRIBUTE::COMPRESSED);
+/// let is_directory  = flags.has(co::FILE_ATTRIBUTE::DIRECTORY);
+/// let is_encrypted  = flags.has(co::FILE_ATTRIBUTE::ENCRYPTED);
+/// let is_hidden     = flags.has(co::FILE_ATTRIBUTE::HIDDEN);
+/// let is_temporary  = flags.has(co::FILE_ATTRIBUTE::TEMPORARY);
+/// # w::SysResult::Ok(())
+/// ```
+#[must_use]
+pub fn GetFileAttributes(file_name: &str) -> SysResult<co::FILE_ATTRIBUTE> {
+	const INVALID: u32 = INVALID_FILE_ATTRIBUTES as u32;
+	match unsafe {
+		ffi::GetFileAttributesW(WString::from_str(file_name).as_ptr())
+	} {
+		INVALID => Err(GetLastError()),
+		flags => Ok(unsafe { co::FILE_ATTRIBUTE::from_raw(flags) }),
+	}
+}
+
+/// [`GetFileAttributesEx`](https://learn.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getfileattributesexw)
+/// function.
+///
+/// This function uses `GET_FILEEX_INFO_LEVELS::GetFileExInfoStandard` flag,
+/// which is the only available flag.
+pub fn GetFileAttributesEx(file: &str) -> SysResult<WIN32_FILE_ATTRIBUTE_DATA> {
+	let mut wfad = WIN32_FILE_ATTRIBUTE_DATA::default();
+	bool_to_sysresult(
+		unsafe {
+			ffi::GetFileAttributesExW(
+				WString::from_str(file).as_ptr(),
+				0,
+				&mut wfad as *mut _ as _,
+			)
+		},
+	).map(|_| wfad)
+}
+
 /// [`GetFirmwareType`](https://learn.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-getfirmwaretype)
 /// function.
 #[must_use]
@@ -396,46 +477,35 @@ pub fn GetLogicalDriveStrings() -> SysResult<Vec<String>> {
 
 	let mut buf = WString::new_alloc_buf(len as usize + 1); // room for terminating null
 
-	bool_to_sysresult(
-		unsafe { ffi::GetLogicalDriveStringsW(len, buf.as_mut_ptr()) } as _,
-	).map(|_| parse_multi_z_str(buf.as_ptr()))
+	unsafe {
+		bool_to_sysresult(
+			ffi::GetLogicalDriveStringsW(len, buf.as_mut_ptr()) as _,
+		).map(|_| parse_multi_z_str(buf.as_ptr(), Some(buf.buf_len())))
+	}
 }
 
-/// [`GetFileAttributes`](https://learn.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getfileattributesw)
+/// [`GetLongPathName`](https://learn.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getlongpathnamew)
 /// function.
-///
-/// # Examples
-///
-/// Checking whether a file or folder exists:
-///
-/// ```no_run
-/// use winsafe::{self as w, prelude::*};
-///
-/// let file_exists = w::GetFileAttributes("C:\\Temp\\test.txt").is_ok();
-/// ```
-///
-/// Retrieving various information about a file or folder path:
-///
-/// ```no_run
-/// use winsafe::{self as w, prelude::*, co};
-///
-/// let flags = w::GetFileAttributes("C:\\Temp\\test.txt")?;
-///
-/// let is_compressed = flags.has(co::FILE_ATTRIBUTE::COMPRESSED);
-/// let is_directory  = flags.has(co::FILE_ATTRIBUTE::DIRECTORY);
-/// let is_encrypted  = flags.has(co::FILE_ATTRIBUTE::ENCRYPTED);
-/// let is_hidden     = flags.has(co::FILE_ATTRIBUTE::HIDDEN);
-/// let is_temporary  = flags.has(co::FILE_ATTRIBUTE::TEMPORARY);
-/// # w::SysResult::Ok(())
-/// ```
 #[must_use]
-pub fn GetFileAttributes(file_name: &str) -> SysResult<co::FILE_ATTRIBUTE> {
-	const INVALID: u32 = INVALID_FILE_ATTRIBUTES as u32;
-	match unsafe {
-		ffi::GetFileAttributesW(WString::from_str(file_name).as_ptr())
+pub fn GetLongPathName(short_path: &str) -> SysResult<String> {
+	let short_path_w = WString::from_str(short_path);
+	let path_sz = match unsafe {
+		ffi::GetLongPathNameW(short_path_w.as_ptr(), std::ptr::null_mut(), 0)
 	} {
-		INVALID => Err(GetLastError()),
-		flags => Ok(unsafe { co::FILE_ATTRIBUTE::from_raw(flags) }),
+		0 => return Err(GetLastError()),
+		len => len,
+	};
+
+	let mut path_buf = WString::new_alloc_buf(path_sz as _);
+	match unsafe {
+		ffi::GetLongPathNameW(
+			short_path_w.as_ptr(),
+			path_buf.as_mut_ptr(),
+			path_sz,
+		)
+	} {
+		0 => Err(GetLastError()),
+		_ => Ok(path_buf.to_string())
 	}
 }
 
@@ -480,7 +550,7 @@ pub fn GetPrivateProfileSection(
 	file_name: &str,
 ) -> SysResult<Vec<(String, String)>>
 {
-	let mut buf_sz = SSO_LEN; // start with no string heap allocation
+	let mut buf_sz = WString::SSO_LEN; // start with no string heap allocation
 	loop {
 		let mut buf = WString::new_alloc_buf(buf_sz);
 		let returned_chars = unsafe { // char count without terminating null
@@ -496,7 +566,7 @@ pub fn GetPrivateProfileSection(
 			return Err(co::ERROR::FILE_NOT_FOUND);
 		} else if (returned_chars as usize) < buf_sz { // to break, must have at least 1 char gap
 			return Ok(
-				parse_multi_z_str(buf.as_ptr())
+				unsafe { parse_multi_z_str(buf.as_ptr(), Some(buf.buf_len())) }
 					.iter()
 					.map(|line| match line.split_once('=') {
 						Some((key, val)) => (key.to_owned(), val.to_owned()),
@@ -540,7 +610,7 @@ pub fn GetPrivateProfileSectionNames(
 	file_name: Option<&str>,
 ) -> SysResult<Vec<String>>
 {
-	let mut buf_sz = SSO_LEN; // start with no string heap allocation
+	let mut buf_sz = WString::SSO_LEN; // start with no string heap allocation
 	loop {
 		let mut buf = WString::new_alloc_buf(buf_sz);
 		let returned_chars = unsafe { // char count without terminating null
@@ -554,7 +624,9 @@ pub fn GetPrivateProfileSectionNames(
 		if GetLastError() == co::ERROR::FILE_NOT_FOUND {
 			return Err(co::ERROR::FILE_NOT_FOUND);
 		} else if (returned_chars as usize) < buf_sz { // to break, must have at least 1 char gap
-			return Ok(parse_multi_z_str(buf.as_ptr()));
+			return Ok(
+				unsafe { parse_multi_z_str(buf.as_ptr(), Some(buf.buf_len())) },
+			);
 		}
 
 		buf_sz *= 2; // double the buffer size to try again
@@ -593,7 +665,7 @@ pub fn GetPrivateProfileString(
 	file_name: &str,
 ) -> SysResult<Option<String>>
 {
-	let mut buf_sz = SSO_LEN; // start with no string heap allocation
+	let mut buf_sz = WString::SSO_LEN; // start with no string heap allocation
 	loop {
 		let mut buf = WString::new_alloc_buf(buf_sz);
 		unsafe { // char count without terminating null
@@ -638,11 +710,23 @@ pub fn GetStartupInfo<'a, 'b>() -> STARTUPINFO<'a, 'b> {
 #[must_use]
 pub fn GetSystemDirectory() -> SysResult<String> {
 	let mut buf = WString::new_alloc_buf(MAX_PATH + 1);
-	bool_to_sysresult(
-		unsafe {
+	let nchars = match unsafe {
+		ffi::GetSystemDirectoryW(buf.as_mut_ptr(), buf.buf_len() as _)
+	} {
+		0 => return Err(GetLastError()),
+		n => n,
+	} as usize;
+
+	if nchars > buf.buf_len() {
+		buf = WString::new_alloc_buf(nchars);
+		if unsafe {
 			ffi::GetSystemDirectoryW(buf.as_mut_ptr(), buf.buf_len() as _)
-		} as _,
-	).map(|_| buf.to_string())
+		} == 0 {
+			return Err(GetLastError());
+		}
+	}
+
+	Ok(buf.to_string())
 }
 
 /// [`GetSystemFileCacheSize`](https://learn.microsoft.com/en-us/windows/win32/api/memoryapi/nf-memoryapi-getsystemfilecachesize)
@@ -1163,6 +1247,7 @@ pub const fn MAKEWORD(lo: u8, hi: u8) -> u16 {
 ///
 /// * [`CopyFile`](crate::CopyFile)
 /// * [`DeleteFile`](crate::DeleteFile)
+/// * [`MoveFileEx`](crate::MoveFileEx)
 /// * [`ReplaceFile`](crate::ReplaceFile)
 pub fn MoveFile(existing_file: &str, new_file: &str) -> SysResult<()> {
 	bool_to_sysresult(
@@ -1175,6 +1260,32 @@ pub fn MoveFile(existing_file: &str, new_file: &str) -> SysResult<()> {
 	)
 }
 
+/// [`MoveFileEx`](https://learn.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-movefileexw)
+/// function.
+///
+/// # Related functions
+///
+/// * [`CopyFile`](crate::CopyFile)
+/// * [`DeleteFile`](crate::DeleteFile)
+/// * [`MoveFile`](crate::MoveFile)
+/// * [`ReplaceFile`](crate::ReplaceFile)
+pub fn MoveFileEx(
+	existing_file: &str,
+	new_file: Option<&str>,
+	flags: co::MOVEFILE,
+) -> SysResult<()>
+{
+	bool_to_sysresult(
+		unsafe {
+			ffi::MoveFileExW(
+				WString::from_str(existing_file).as_ptr(),
+				WString::from_opt_str(new_file).as_ptr(),
+				flags.raw(),
+			)
+		},
+	)
+}
+
 /// [`MulDiv`](https://learn.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-muldiv)
 /// function.
 #[must_use]
